VJplotFormantTriangleGrid <- function(x_, myTitle='', myTitleTail='', plotEllipse=FALSE, plotTriangle='no', plotPoints=TRUE, EllipseinclusionRate=0.5, job='plot') {
    #x_ <- myFrmnts2Plt
    x <- copy(x_)
    x <- x[(PltIndx) & !is.na(myF1b) & !is.na(myF2b)]
    
    # now test on the nasty situation that there may be to many distinct samples, restrict to max 15
    tmp <- x[,.N,by=sample][order(N, decreasing = TRUE)]
    tmp[, rnk_ := 1:tmp[,.N]]
    x <- merge(x, tmp[,.SD,.SDcols=c('sample','rnk_')], all.x = TRUE)
    x[, sample := factor(ifelse(rnk_ < 15, sample, '~other'))]
    
  myPlot <- ggplot(data=x, aes(x=myF2b, y=myF1b, color=sample, shape=sample)) +
    theme_bw() +
    labs(shape = "sample") +
    labs(color = "sample") +
    scale_shape_manual(values=rep(1:14,2)) +
    scale_x_continuous(limits = c(18,4 ), trans='reverse', name='F2 [bark]', sec.axis = sec_axis(~ bark2f(.), name='F2 [Hz]', breaks=c(500, 1000,2000,3000)))+
    scale_y_continuous(limits = c(10,2), trans='reverse', name='F1 [bark]', sec.axis = sec_axis(~ bark2f(.), name='F1 [Hz]', breaks=c(250,500,750,1000))) +
    facet_wrap(~tag) +
    ggtitle(paste0(myTitle,': F1/F2 (', myTitleTail,')'))

  if (plotTriangle == 'yes') {
    myPlot <- myPlot + 
      annotate("segment", y=2.47, x=14.9, yend=2.77, xend=5.4, linetype='dashed') + annotate("segment", y=2.77, x=5.4, yend=8.4, xend=10.7, linetype='dashed') + annotate("segment", y=8.4, x=10.7, yend=2.47, xend=14.9, linetype='dashed') +
      annotate("segment", y=2.54, x=16.7, yend=2.90, xend=5.2, linetype='dotted') + annotate("segment", y=2.90, x=5.2, yend=9.6, xend=11.3, linetype='dotted') + annotate("segment", y=9.6, x=11.3, yend=2.54, xend=16.7, linetype='dotted')
  } 

  if (plotPoints) {
    myPlot <- myPlot + geom_point()
  }
  
  if (plotEllipse) {
    # https://github.com/JoFrhwld/FAAV/blob/master/r/stat-ellipse.R
    # https://en.wikipedia.org/wiki/Confidence_region
    # get the level%-inclusion confidence region of p=2 * n data 
    # QF: quantile function of F-distribution
    # QF(1-alpha, p, n-p) = QF(level, p, n-p)
    unitCircle <- t(sapply(seq(from=0, to=2*pi, length.out = 50), FUN=function(x) c(cos(x),sin(x))))
    myEllipses <- do.call(rbind, lapply( split(x, x$tag), function(mySplitByTag) {
      do.call(rbind, lapply( split(mySplitByTag, mySplitByTag$sample), function(mySplitBySample) {
        if(mySplitBySample[,.N]>=3) {
          tmp <- cov.trob( mySplitBySample[,.(myF1b,myF2b)] ) 
          tmp$scaleEllipsoid <- with(tmp, sqrt(2*qf(EllipseinclusionRate, 2, n.obs)))
          myEllipse <- as.data.table(t(with(tmp, center + scaleEllipsoid*t(unitCircle %*% chol(cov)))))
          myEllipse[, c('tag','sample','centerF1', 'centerF2') := .(mySplitBySample[1,]$tag, mySplitBySample[1,]$sample, tmp$center[1], tmp$center[2])]
          return(myEllipse)
        }
      }))
    }))
    myPlot <- myPlot +  
      geom_polygon(data=myEllipses, aes(color=sample, fill=sample), alpha=0.1) +
      geom_text(data=myEllipses[,.SD[1,],by=.(tag,sample)], aes(x=centerF2, y=centerF1, label=sample))
  }

  if (job=='plot') {
    print(myPlot)
  
    # -- get sizing
    tmp <- length(unique(x$tag))
    myScale <- ifelse(tmp>6,1,1/(6-tmp))
    # -- plot
    ggsave(paste0('VJplotF12_',myTitle,'_',myTitleTail,'.png'), dpi=300, width=29.7*myScale, height=21*myScale)
  } else if (job=='return') {
    return(myPlot)
  }
}

VJplotSpectrum <- function (x_, job='plot') {
  # #NB: protect to pick up only 1st record = single sample plot
  x <- x_[1,]
  #print(x)
  tf <- as.numeric(x[, .SD, .SDcols=c('tf')])
  F <- as.numeric(x[, .SD, .SDcols=paste0('F',1:3,'P')])
  B <- as.numeric(x[, .SD, .SDcols=paste0('B',1:3,'P')])
  
  # Omega, VJ p. 444 (44). ceiling frequency * 2 = sampling rate
  O <- 2*pi*F/tf/2 
  # r, VJ p. 444 (45)
  r <- exp(- B * pi/tf/2 )
  
  # points on frequency contour
  N <- 250
  # frequency contour
  n0 <- round(200/tf*N) # start with 200 Hz
  fc <- (n0:N)/N*tf 
  # frequency on unit cirlce
  ufc <- complex(argument = (n0:N)/N*pi)
  
  # LPC frequency response; VJ (43)
  vzbase <- function(z) {
    do.call(sum, 
            lapply(1:3,
                   FUN=function(i) -20*log(abs((1 - complex(modulus = r[i], argument = O[i])*z^-1 ) * (1 - complex(modulus = r[i], argument = -O[i])*z^-1 )))
            )  
    )
  }
  # preemphase filter; VJ (49)
  mu <- 0.9
  pz <- -20*log(abs(1 - mu * ufc^-1))
  
  # preemphased frequency response
  #fr <- apply(rbind(sapply(ufc, vzbase),pz), 2, sum)
  fr <- sapply(ufc, vzbase)
  
  spectrum <- as.data.table(cbind(fc,fr))
  spectrum[,fcb:=f2bark(fc)]
  
  formants <- as.data.table(cbind(n=1:length(F),F))
  formants[,fr:=sapply(F, FUN=function(f) vzbase(complex(argument = f/tf*pi)))]
  
  myTitle <- x[,paste0(tag,' (',sampleID,') ', sample)] 
  
  MyPlot <- ggplot(data=spectrum, aes(x=fcb, y=fr)) +
    theme_bw() +
    theme(legend.position = 'none', panel.grid = element_blank()) +
    geom_line(size=1.5) +
    geom_vline(data=formants, aes(xintercept=f2bark(F), color=factor(n)) ) +
    geom_text(data=formants, aes(x=f2bark(F), label=paste0(signif(F,2),' Hz'), color=factor(n)), hjust=-0.1, vjust=-0.1 ) +
    geom_text(data=formants, aes(x=f2bark(F), y=spectrum[,min(fr)], label=paste0('F',n), color=factor(n)), hjust=1.3, vjust=-0.1 ) +
    scale_x_continuous(name='frequency [bark]', sec.axis = sec_axis(~ bark2f(.), name='[Hz]')) +
    ylim(spectrum[,min(fr)], spectrum[,max(fr)]*1.1) +
    ylab('vocal tract frequency response [db]') +
    ggtitle(myTitle)
  
  if (job=='plot') {
    print(MyPlot)
    ggsave(paste0('VJplotSpctrm_',x$sampleID,'.png'), dpi=300, width=10, height=10, units = 'cm')
  } else if (job=='return') {
    return(MyPlot)
  }
}

VJplotStatistics <- function(x, myTitle = '', myTitleTail='') {
  #x <- myFrmnts2Plt[tag=='Ritter']
  stats <- x[(PltIndx) & !is.na(myF1b) & !is.na(myF2b), .(N=.N, mF1b=mean(myF1b), mF2b=mean(myF2b), sdF1b=sd(myF1b), sdF2b=sd(myF2b)), by=.(tag, sample)]
  stats[, c('mF1h','mF2h', 'sdF1h', 'sdF2h') := .(bark2f(mF1b), bark2f(mF2b), bark2f(sdF1b), bark2f(sdF2b))]
  stats[, c('sdrF1h', 'sdrF2h') := .(signif(sdF1h, digits=1), signif(sdF2h, digits=1))]
  stats[, c('dgtF1h', 'dgtF2h') := .(-floor(log10(sdrF1h)), -floor(log10(sdrF2h)))]
  stats[, c('mrF1h', 'mrF2h') := .(round(mF1h, dgtF1h), round(mF2h, dgtF2h))]
  
  stats <- merge(stats[,.(tag, sample, N, mF1b, mF2b, sdF1b, sdF2b, mrF1h, mrF2h, sdrF1h, sdrF2h)], myVwlOrdr, all.x=TRUE, by.x='sample', by.y='VV')
  
  print(
    ggplot(data=stats, aes(x=mF2b, y=OO, color=factor(O))) +
      theme_bw() +
      theme(legend.position = 'none')+
      geom_segment(aes(x=mF2b-sdF2b, xend=mF2b+sdF2b, yend=OO))+
      geom_point() +
      geom_text(aes(label=sample, x=mF2b), vjust=-0.4, size=4) +
      geom_text(aes(label=paste0(mrF2h,'\u00B1',sdrF2h,' n=',N), x=mF2b-sdF2b), vjust=1.4, hjust=0.5, size=3) +
      facet_wrap(~PP, nrow=1) +
      scale_x_continuous(limits = c(22,1 ), trans='reverse', name='F2 [bark; reports: F2-mean (dot) \u00B1 F2-sDev (bar)]', sec.axis = sec_axis(~ bark2f(.), name='[Hz]', breaks=c(500,1000,2000,4000))) +
      ylab('Openness') +
      ggtitle(paste0(myTitle, ': F2 statistics ',myTitleTail))
  )
  ggsave(paste0('VJstatsF2_',myTitle,'_',myTitleTail,'.png'), dpi=300, width=24, height=12, units='cm')
  
  print(
    ggplot(data=stats, aes(y=mF1b, x=OO, color=factor(O))) +
      theme_bw() +
      theme(legend.position = 'none', axis.text.x=element_text(angle=90, hjust=1, vjust=0.5))+
      geom_segment(aes(y=mF1b-sdF1b, yend=mF1b+sdF1b, xend=OO))+
      geom_point() +
      geom_text(aes(label=sample, y=mF1b-sdF1b), hjust=0.5, vjust=-0.5, size=4) +
      geom_text(aes(label=paste0(mrF1h,'\u00B1',sdrF1h,' n=',N), y=mF1b+sdF1b), vjust=0.5, hjust=1.1, size=3, angle=90) +
      facet_wrap(~PP, scales='free_x', nrow=1)    +
      scale_y_continuous(limits = c(10,2 ), trans='reverse', name='F1 [bark]', sec.axis = sec_axis(~ bark2f(.), name='[Hz]', breaks=c(250, 500,1000))) +
      xlab('Openness [reports: F1-mean (dot) \u00B1 F1-sDev (bar)]') +
      ggtitle(paste0(myTitle, ': F1 statistics ',myTitleTail))
  )
  ggsave(paste0('VJstatsF1_',myTitle,'_',myTitleTail,'.png'), dpi=300, width=24, height=12, units='cm')
}

# x <- myFrmnts2Plt[tag=='TSch_TS']
# 0.15: adding debug level, protecting T2 kernel from low sample sizes
VJanalyzeDifference <- function(x, myTitle = '', myTitleTail='', mode='byTag', debugLevel=0) {
  z <- x[(PltIndx) & !is.na(myF1b) & !is.na(myF2b) ]
  if (debugLevel>=1) { print(z[1,]$tag)}
  # apply Hotteling's T2 test
  pvalues <- do.call(rbind, 
                     lapply(
                       split(z, z$tag), FUN=function(x) { # this seems to make no sense, we already enter here split by tag???
                         do.call(rbind, 
                                 lapply(
                                   combn(unique(x$sample),2, simplify = FALSE), FUN=function(y) {
                                     ret <- data.table(tag1=x[sample==y[1]][1,]$tag, tag2=x[sample==y[2]][1,]$tag, s1=y[1], s2=y[2], N1=x[sample==y[1], .N], N2=x[sample==y[2], .N])
                                     if (debugLevel>=2) { print( paste0(y[1],' n=', ret$N1, ' / ',y[2], ' n=', ret$N2)) }
                                     # myFrmnts2Plt[sample=='y:' & tag=='BID2_VJ']
                                     # myFrmnts2Plt[sample=='2:' & tag=='BID2_VJ']
                                     
                                     if (ret$N1<3 | ret$N2<3 ) {
                                       # low sample size
                                       ret[, p:= NA]
                                     } else {
                                       ret[, p:= HotellingsT2( x[sample==y[1], .(myF1b, myF2b)], x[sample==y[2], .(myF1b, myF2b)])$p.value  ]
                                     }
                                     # 
                                     # #print(paste0(y[1],' n=', x[sample==y[1], .N],'; ',y[2], ' n=',x[sample==y[2], .N], ' _ p=', tmp$p.value))
                                     # return(data.table(tag1=x[sample==y[1]][1,]$tag, tag2=x[sample==y[2]][1,]$tag, s1=y[1], s2=y[2], , .N], p=as.numeric(tmp)))
                                     return(ret)
                                   })
                         )
                       })
  )
  pvalues[,slevel := VJsignificance(p)]
  pvalues[,slevel := factor(ifelse(N1<3 | N2<3, 'ls', as.character(slevel)), levels=levels(pvalues$slevel))] # adjust in case of low sample counts
  pvalues[,s1 := factor(s1, levels=levels(myVwlOrdr$VV))]
  pvalues[,s2 := factor(s2, levels=levels(myVwlOrdr$VV))]
  
  print(
    ggplot(data=pvalues, aes(x=s1, y=s2, color=slevel, size=slevel, shape=slevel)) +
      theme_bw() +
      labs(size='signif.', shape='signif.', color='signif.') +
      scale_shape_manual(values=c(4,2,17,15,15,47))+
      scale_size_manual(values=c(1,2,2,2,3,3))+
      scale_color_manual(values=c('blue','blue','blue','darkgreen','darkgreen','black'))+
      geom_point() +
      xlab('Significance of centroid differnces') +
      ylab('(Hotelling T2 test)') +
      ggtitle(paste0(myTitle,myTitleTail))
  )
  ggsave(paste0('VJsgnf_',myTitle,'_',myTitleTail,'.png'), units = 'cm', width=12, height=12, dpi=300)  
}
