VJBackProjectFormant <- function(Z, formant='F1', method='robustLobanov') {
  # Z -> formants Hz; VJ, p. 128, tab. 11 (re-read from VJ base, 3 significant)
  # tmp <- summaryBy( data=subset(AllQuantileStats, Group=='HG1' & (measure=='QMean'|measure=='Qsd'|measure=='MAD'|measure=='P50')
  #                               & (var=='F1pT'|var=='F2pT'|var=='F3pT'|var=='F4pT'|var=='B1p'|var=='B2p'|var=='B3p'|var=='B4p')), original~key, FUN=mean )
  # tmp$run <- 'all'
  # RsltNrmPrjctn <- dcast(data=tmp, run~key, value.var='original.mean') 
  # signif(RsltNrmPrjctn[2:ncol(RsltNrmPrjctn)],3)
  # B1p.MAD B1p.P50 B1p.QMean B1p.Qsd B2p.MAD B2p.P50 B2p.QMean B2p.Qsd B3p.MAD B3p.P50 B3p.QMean B3p.Qsd B4p.MAD B4p.P50 B4p.QMean B4p.Qsd F1pT.MAD F1pT.P50 F1pT.QMean F1pT.Qsd F2pT.MAD F2pT.P50
  # 1   0.739    1.19      2.01     0.9   0.865    1.72      2.85     1.2    1.22    2.38      3.88     1.6    1.58    2.89      4.71    1.97     1.21     4.42       5.08     1.13     1.65     11.5
  # F2pT.QMean F2pT.Qsd F3pT.MAD F3pT.P50 F3pT.QMean F3pT.Qsd F4pT.MAD F4pT.P50 F4pT.QMean F4pT.Qsd
  # 1       10.5     1.93    0.397     14.8       14.7    0.522    0.301     16.8       16.6    0.403
  myBckPrjctn <- rbind(
    data.table(msre='F1', trgtGndr='androgyn', QntleWdthNrmlzed=1.13, QntleMdPnt=5.08, md=4.42, mad=1.21),
    data.table(msre='F2', trgtGndr='androgyn', QntleWdthNrmlzed=1.93, QntleMdPnt=10.5, md=11.5, mad=1.65),
    data.table(msre='F3', trgtGndr='androgyn', QntleWdthNrmlzed=0.522, QntleMdPnt=14.7, md=14.8, mad=0.397),
    data.table(msre='F4', trgtGndr='androgyn', QntleWdthNrmlzed=0.403, QntleMdPnt=16.6, md=16.8, mad=0.301),
    
    data.table(msre='B1', trgtGndr='androgyn', QntleWdthNrmlzed=0.9, QntleMdPnt=2.01, md=1.19, mad=0.739),
    data.table(msre='B2', trgtGndr='androgyn', QntleWdthNrmlzed=1.2, QntleMdPnt=2.85, md=1.72, mad=0.865),
    data.table(msre='B3', trgtGndr='androgyn', QntleWdthNrmlzed=1.6, QntleMdPnt=3.88, md=2.38, mad=1.22),
    data.table(msre='B4', trgtGndr='androgyn', QntleWdthNrmlzed=1.97, QntleMdPnt=4.71, md=2.89, mad=1.58)
  )
  myKey <- regmatches(formant, regexpr("[BF][1234]", formant))
  if (length(myKey)>0) {
    myPrms <- myBckPrjctn[msre==myKey]
    if (method=='robustLobanov') {
      w <- myPrms$QntleWdthNrmlzed
      m <- myPrms$QntleMdPnt
    } else if (method=='robustZ') {
      w <- myPrms$mad
      m <- myPrms$md
    } else {
      w <- 1
      m <- 0
    }
    return( bark2f( Z * w + m) ) # VJ, p. 92, (31)
  } else {
    return(Z)
  }
}

VJquantileStatistics <- function(x, BandStart=0.05, BandWidth=0.04, y=NULL) {
  myPrbs <- c(BandStart-BandWidth, BandStart, 1-BandStart, 1-BandStart+BandWidth)
  
  # --- restrict series to analyze
  x_ <- x[!is.na(x)]
  # --- in case we have F1 (y) to control F2 (x) ...
  if (!is.null(y)) {  x_ <- x[y<quantile(y, probs=0.5, na.rm = TRUE) & y>quantile(y,probs=0.01, na.rm=TRUE)] }
  # --- actual quantiles
  Qntle <- quantile(x_, probs=myPrbs, na.rm=TRUE)
  # --- lower quantile mean
  if (length(x_[x_ >= Qntle[1] & x_ <= Qntle[2] ]) > 0) {
    QntleMnLwr <- mean( x_[x_ >= Qntle[1] & x_ <= Qntle[2]] )
  } else {
    QntleMnLwr <- quantile(x_, probs=(BandStart-BandWidth+BandStart)/2, na.rm=TRUE) # interpolated mid
  }
  # --- upper quantile mean
  if (length(x_[x_ >= Qntle[3] & x_ <= Qntle[4] ]) > 0) {
    QntleMnUppr <- mean( x_[x_ >= Qntle[3] & x_ <= Qntle[4]] )
  } else {
    QntleMnUppr <- quantile(x_, probs=(1-BandStart+1-BandStart+BandWidth)/2, na.rm=TRUE) # interpolated mid
  }
  # === now we need to calculate correction to standard normal, VJ, pp. 89-90
  # --- calculate empirical distribution function x_ -> [0,1]
  myECDF <- ecdf(x_) 
  # --- look up where there aforecalculated quantile means lie on the ECDF [0,1]
  QQAvg <- myECDF( c(QntleMnLwr, QntleMnUppr) )
  # handling rubbishly small populations
  QQAvg <- c( max(BandStart-BandWidth,QQAvg[1]), min(1-BandStart+BandWidth,QQAvg[2]))
  # --- calculate correction factor
  k <- qnorm(QQAvg[2])-qnorm(QQAvg[1])
  return(list(
    BndStrt=BandStart,
    BndWdth=BandWidth,
    QntleMnLwr=QntleMnLwr, # (21) lower quantile mean
    QntleMnUppr=QntleMnUppr, # (22) uper quntile mean
    QntleWdth=QntleMnUppr-QntleMnLwr, # (23) quantile width
    QntleWdthNrmlzed=(QntleMnUppr-QntleMnLwr)/k, # (24) quantile width, N-0 normalized
    QntleMdPnt=(QntleMnUppr+QntleMnLwr)/2, # (25) quantile midpoint
    m=mean(x_, na.rm=TRUE),
    md=median(x_, na.rm=TRUE),
    sd=sd(x_, na.rm=TRUE),
    mad=mad(x_, na.rm=TRUE),
    min=min(x_, na.rm=TRUE),
    max=max(x_, na.rm=TRUE)
  ))
}

# bark conversion convenience wrapper
VJbcw_ <- function(x, c2b=TRUE) {
  if (c2b) {
    f2bark(x)
  } else {
    x
  }
}

# --- calc for (pre-)selected series, but in context
VJnormalizeCalcWrapper <- function(dt, Srs2Nrmlze='F2P', method='robustLobanov', convertToBark=TRUE, BandStart=0.05, BandWidth=0.04, toReturn='add.series', doBackProjection='yes', MaxFilterError=0.9) {
  # --- calculate quantile statistics, in case
  y <- NULL
  # if we intend to normalize F2-ish series, the correspondend F1 series becomes the control series 
  if( grepl('F2',Srs2Nrmlze) ) {  
    tmp <- sub('F2','F1',Srs2Nrmlze)
    y <- VJbcw_(dt[(NrmlztnRfrnce), get(tmp) ], c2b=convertToBark)
  }
  QntleSts <- VJquantileStatistics( VJbcw_(dt[(NrmlztnRfrnce), get(Srs2Nrmlze)], c2b=convertToBark), y=y, BandStart=BandStart, BandWidth=BandWidth)
  QntleSts$Srs <- Srs2Nrmlze
  QntleSts$Mthd <- method
  QntleSts$Cnvrt2Brk <- convertToBark 
  if (toReturn=='add.series') {
    if (method=='robustLobanov') { # (29)
      nx <- ( VJbcw_(dt[,get(Srs2Nrmlze)], c2b=convertToBark) - QntleSts$QntleMdPnt) / QntleSts$QntleWdthNrmlzed
      if (convertToBark & doBackProjection=='yes') { nx <- VJBackProjectFormant(nx, formant = Srs2Nrmlze, method='robustLobanov')  }
    } else if (method=='robustZ') { # (27)
      nx <- ( VJbcw_(dt[,get(Srs2Nrmlze)], c2b=convertToBark) - QntleSts$md) / QntleSts$mad
      if (convertToBark & doBackProjection=='yes') { nx <- VJBackProjectFormant(nx, formant = Srs2Nrmlze, method='robustZ')  }
    } else if (method=='robustGerstman') { # (26)
      nx <- ( VJbcw_(dt[,get(Srs2Nrmlze)], c2b=convertToBark) - QntleSts$QntleMnLwr) / ( QntleSts$QntleMnUppr - QntleSts$QntleMnLwr )
    } else if (method=='Lobanov') { # (16)
      nx <- ( VJbcw_(dt[,get(Srs2Nrmlze)], c2b=convertToBark) - QntleSts$m) / QntleSts$sd
    } else if (method=='Gerstman') { # (25)
      nx <- ( VJbcw_(dt[,get(Srs2Nrmlze)], c2b=convertToBark) - QntleSts$min) / (QntleSts$max - QntleSts$max)
    }
    rtrn <- data.table(nx=nx)
    setnames(rtrn, paste0(Srs2Nrmlze,'N'))
    return(rtrn)
  } else if (toReturn=='statistics') {
    return(as.data.table(QntleSts))
  } else {
    NULL
  }
}

VJnormalizeDoSeries <- function(dt_, AllSrs2Nrmlze='F2P', method='robustLobanov', convertToBark=TRUE, doBackProjection='yes', BandStart=0.05, BandWidth=0.04, MaxFilterError=0.9) {
  dt <- copy(dt_)
  dub <- lapply(AllSrs2Nrmlze, function(x) dt[, VJnormalizeCalcWrapper(.SD, Srs2Nrmlze=x, method=method, convertToBark=convertToBark, doBackProjection=doBackProjection, BandStart=BandStart, BandWidth=BandWidth, toReturn='add.series', MaxFilterError=MaxFilterError)])
  null_ <- lapply(dub, function(x) dt[, names(x) := x])
  return(dt)
}

VJnormalizeGetStatistics <- function(dt, AllSrs2Nrmlze='F2P', method='robustLobanov', convertToBark=TRUE, doBackProjection='yes', BandStart=0.05, BandWidth=0.04, castBack=TRUE, MaxFilterError=0.9) {
  tmp <- do.call(rbind, lapply(AllSrs2Nrmlze, function(x) dt[, VJnormalizeCalcWrapper(.SD, Srs2Nrmlze=x, method=method, convertToBark=convertToBark, BandStart=BandStart, BandWidth=BandWidth, toReturn='statistics', MaxFilterError=MaxFilterError)]))
  # for convenience we cast back from bark to Herz
  if (castBack) {
    doClmns <- colnames(tmp)[3:(ncol(tmp)-3)]
    tmp[, paste0(doClmns,'f') := lapply(.SD, FUN=bark2f), .SDcols=doClmns]
  }
  return(tmp)
}

