VJmyDTT3 <- function(z, N, k) {
  DTT3	<- 2/N * rbind( 0.5*z[1]*rep(1,N) , sapply( 0:(N-1), function (x) {z[(2:N)] * cos( pi/N * (1:(N-1)) * ( x+0.5  ) ) } ))
  sapply(1:N, function (y) sum(DTT3[1:k,y]))
}

# --- take in (F1,F2,F3,B1,B2,B3) series, DTT it and calculate pass error
VJformantDTT <- function (x_, 
                          myCushion = 0.2,
                          myMaxOrder = 3,
                          myWeightBandwith = 0,
                          myGenderHintMalus = 0.3, # per kHz off 5000 male, 5500 female 
                          myPickSingle = 0.3, # standard, VJ, p. 116
                          myPickPair = 0.2,
                          signOfLive = -1,
                          myF3errorWeight = 0.75,
                          myLocationProbabilityWeight = 0.1,
                          myLocationProbabilityWidth = 0.5, # area/probability of central area with resulting pseudo probability weight 1 
                          myTriangleTest = 'none', # male/female/general/none 
                          myTriangleCreep = 0.25, # enhance VJ triangle by this in bark
                          returnSeries = FALSE
) {
  #print(myCushion)
  # x_ <- myRwFrmnts[myKeySweep == 'Ritter~100725~5800']         
  myReturnFields <- grep('[FB\\d]', colnames(x_), value=TRUE, invert = TRUE) # dont return the sampe formant/bandwidth values
  x <- copy(x_) # nasty, to break .SD tie http://www.forum.r-statistik.de/viewtopic.php?p=5112
  N0 <- nrow(x)
  krnlIndxLft <- round(myCushion*N0)
  krnlIndxRght <- round((1-myCushion)*N0)
  krnlIndx <- (krnlIndxLft+1) : (krnlIndxRght-1)
  N <- length(krnlIndx)
  myInptFrmnts <-  c('F1','F2','F3')
  myInptBndWdth <- c('B1','B2','B3')
  myInpt <- c(myInptFrmnts,myInptBndWdth)
  
  # --- convert to Bark
  x[, paste0(myInpt,'b') := lapply(.SD, f2bark), .SDcols=myInpt] 
  # --- DTT params
  x[krnlIndx, paste0('DTT2',myInpt) := lapply(.SD, dtt, type='dct', variant=2), .SDcols=paste0(myInpt,'b')] 
  # --- DTT filtering
  x[krnlIndx, paste0(myInpt,'bFltrd') := lapply(.SD, VJmyDTT3, N=.N, k=myMaxOrder), .SDcols=paste0('DTT2',myInpt)] 
  
  if (myTriangleTest=='yes') {
    tmp <- VJgetTriangle(gender=x[1,]$GndrUse, creep=myTriangleCreep)
    x[krnlIndx, Trngle := VJtestInsideTriangle( F1bFltrd, F2bFltrd, xTrngle=tmp$xTrngle, yTrngle=tmp$yTrngle) ]
  } else {
    x[krnlIndx, Trngle := 1]
  }
  
  # --- pick up points
  x[, c('pckP','pckL','pckR') := 0]
  x[krnlIndxLft + myPickSingle*N + 1, pckP := 1 ]
  x[krnlIndxLft + myPickPair*N + 1, pckL := 1 ]
  x[krnlIndxLft + (1-myPickPair)*N + 1, pckR := 1 ]
  # --- in case of filtered series to be returned (in Hertz)
  if (returnSeries) { 
    myReturn <- x
    myReturn[, (paste0(myInpt,'Fltrd')) := x[, lapply(.SD, FUN=bark2f), .SDcols = paste0(myInpt,'bFltrd')] ]
  } else {
    myReturn <- x[pckP==1, ..myReturnFields]
  }
  # --- return extracted values, backconverted to Hertz
  myReturn[, (paste0(myInpt,'P')) := x[pckP==1, lapply(.SD, bark2f), .SDcols = paste0(myInpt,'bFltrd')] ]
  myReturn[, (paste0(myInptFrmnts,'L')) := x[pckL==1, lapply(.SD, bark2f), .SDcols = paste0(myInptFrmnts,'bFltrd')] ] # formants only
  myReturn[, (paste0(myInptFrmnts,'R')) := x[pckR==1, lapply(.SD, bark2f), .SDcols = paste0(myInptFrmnts,'bFltrd')] ] # formants only
  
  # === Get Filter Error
  # --- Filter Errors DTT
  myFltrErrrCmpnts <- sapply(myInpt, function(Inpt) x[krnlIndx, sum(abs(get(paste0(Inpt,'b'))-get(paste0(Inpt,'bFltrd')))) / mean(get(paste0(Inpt,'b'))) /.N ])
  FltrErrrBse <- 10 * log( (1-myWeightBandwith)*sum(myFltrErrrCmpnts[1:3]*c((1-myF3errorWeight)/2,(1-myF3errorWeight)/2,myF3errorWeight)) + 
      myWeightBandwith*sum(myFltrErrrCmpnts[4:6]*c((1-myF3errorWeight)/2,(1-myF3errorWeight)/2,myF3errorWeight) ))
  myReturn[, FltrErrrBse := FltrErrrBse]
  
  # --- GenderMalus
  if (myGenderHintMalus>0) {
    if (x[1,]$GndrUse == 'female' & x[1,]$tf<5500) {
      GndrMlsFctr <- 1-abs(x[1,]$tf - 5500)/1000*myGenderHintMalus
    } else if (x[1,]$GndrUse == 'male'& x[1,]$tf>5000 ) {
      GndrMlsFctr <- 1-abs(x[1,]$tf - 5000)/1000*myGenderHintMalus
    } else {
      GndrMlsFctr <- 1
    }
  } else {
    GndrMlsFctr <- 1
  }
  myReturn[, GndrMlsFctr := GndrMlsFctr]
  
  # --- in triangle rate
  InTrngleRte <- x[krnlIndx, sum(Trngle)/.N]
  myReturn[, InTrngleRte := InTrngleRte]
  
  # --- pseudo location probabilty
  #myStats <- myFstats[phoneme==x[1,sample] & gender==x[1,GndrUse]]
  if (!is.na(x[1,]$F1P50)) {
    myz <- qnorm(0.5 + myLocationProbabilityWidth/2)
    myzf1 <- with(x[1,],(f2bark(myReturn$F1P)-F1P50)/((F1P75-F1P25)/2/0.67))
    myzf2 <- with(x[1,],(f2bark(myReturn$F2P)-F2P50)/((F2P75-F2P25)/2/0.67))
    F1Prbblty <- ifelse( abs(myzf1) < myz, 1, dnorm(myzf1)/dnorm(myz)) 
    F2Prbblty <- ifelse( abs(myzf2) < myz, 1, dnorm(myzf2)/dnorm(myz))
    FPrbblty <- F1Prbblty*F2Prbblty
  } else {
    F1Prbblty <- 1
    F2Prbblty <-1
    FPrbblty <- 1
  }
  myReturn[, c('F1Prbblty','F2Prbblty','FPrbblty') := .(F1Prbblty,F2Prbblty,FPrbblty)]
  
  # --- total error
  myReturn[, FltErr := GndrMlsFctr * InTrngleRte * (1-myLocationProbabilityWeight+myLocationProbabilityWeight*FPrbblty) * FltrErrrBse ]
  
  if(signOfLive>0) {
    tmp <- x$i_ %% signOfLive
    if(length(tmp[tmp==0])>0) { message(x$i_[which(tmp==0)[1]],' ', appendLF = FALSE)}
  } 
  return(myReturn)
}

VJmarkWinner <- function(x, 
                         selectBand=0.02 # include all measurments near min error, VJ, p. 107, pt. 8
) {
  # break ties, should they exist, by adding minor noise
  x$FltErr <- x$FltErr * (1-runif(n=nrow(x))/1000)
  # get min, mark and return
  myMin <- min(x$FltErr)
  x$FrmntSlct <- 0 # to be deselected
  x[FltErr == myMin, FrmntSlct := 1] # 1 marks winner
  x[FltErr <= myMin*(1-selectBand) & FrmntSlct != 1 & abs(tf- x[FrmntSlct == 1]$tf)<500, FrmntSlct := 2] # 2 other within band range, in case
  return(x)
}

VJextractWinner <- function(x_) {
  # x is result set from 
  x <- copy(x_)
  myCols <- grep('[FB]\\d[PLR]', colnames(x), value = TRUE) # get formant, bandwidth and intensity values for later averaging
  tmp <- x[ FrmntSlct>=1, lapply(.SD, mean, na.rm=TRUE), .SDcols=myCols] # average 
  x[FrmntSlct==1, (myCols) := tmp] #store in winner as carrier record
  x$FrmntSlctN <- x[ FrmntSlct>=1, .N] # use the FrmntSlct field as carrier for count of records 
  return(x[FrmntSlct==1][1,]) # protect the 0db case where we only have rubbish and return 1st record
}
