form getParameters
	sentence  	myWavDirectory			; Input WAV file directory, evtl. including full path.
	word	  	myWavName				; Input WAV file name, but excluding file extension '.wav'.
	word		myWavExtension			; Now, the (sound) file may actually not be of type 'wav' (default) but 'mp3' and other also
	sentence  	myTextGridDirectory		; Input PRAAT text grid directory, evtl. including full path.
	word		myTextGridName			; Input PRAAT text grid file name, but excluding file extension '.textgrid'.
	sentence  	myOutputdirectory		; Output directory, evtl. including full path.
	word      	tagOut					; The tag to write into the file and filename. Note that all samples must have different tags (or offset number ranges - see below), when later processed together in the R-kernel.
	word		tailTagOut				; This is to seperate output files within a multi-thread environment, where one sample/speaker (=tagOut) was spread over multiple threads. This is "just" a tailing tag for the filename, the subsequent R-kernel will merge all within one speaker/sample (=tagOut) together (default: 1).
	integer		theChannel				; Input WAV channel to go for (default: 1).
	integer 	shutterTier				; This PRAAT text grid tier is the phonem tier and defines the time segments resp. the phonems we calculate for (default: 1). Only non-empty segments are processed and usally. The phonem tier should contain phonems coded in generalized SAMPA letter codes [for vowels, VJ: 'S2' in tab. 2, p. 21], e.g. 'a:' (see as well filterSet, below).
	integer 	annotationTier			; This PRAAT text grid tier is the annotation tier, usually the full word from which the phonem is taken, e.g. 'Tagesschau' (default: 2). If not required, use same as shutterTier.
	integer 	offset					; Offset for internal ID numbering (if not required, use: 0). This assigns different number ID-ranges to different samples / speakers. Note, that all samples must have different offset number ranges (or tags - see above), when later processed together in the R-kernel.
	integer		lowerFrequency			; Formant frequency ceiling in PRAAT: start value of sweep in Hz (default: 4000) [VJ: pp. 64-71].
	integer		upperFrequency			; Formant frequency ceiling in PRAAT: end value of sweep in Hz (default: 6000) [VJ: pp. 64-71].
	integer		frequencyStepWidth		; Formant frequency ceiling in PRAAT: increased in sweep by that amount in Hz (default: 100) [VJ: pp. 64-71].
	integer		windowLength			; Window length in msec, default: 25 [VJ: p. 36].
	integer		windowShift				; Window shift in msec, default: 2 [VJ: p. 36].
	integer		maxCalcs				; Set to -1 to disable any limits (default), else calculation will terminate after here specifified number of phonems calculated.
	word		filterSet				; Restrict calculations to certain type of phonems pursuant on what is found in the phonem tier. 'none': No restriction are applied (default). 'vowels': Restrict to vowels - the phonem has to start with a generalized SAMPA vowel letter code [VJ: 'S2' in tab. 2, p. 21], what will catch diphthongs as well. 'consonants': Restrict to non-vowels (i.e. anything else).
	integer		delayStart				; number of seconds thh script waits before it actually starts, what is required to keep messages somehow tidy in multithreading environment (default: 0).
endform

sleep(delayStart)

clearinfo

myHeader$ = tagOut$ + "_" + tailTagOut$ + ": "

writeInfoLine(  myHeader$ + "")
writeInfoLine(  myHeader$ + "=== VokalJaeger Enhanced Algorithmic Toolpack: VJcalcFormants.praat")
writeInfoLine(	myHeader$ + "- author : Carsten Keil (http://vokaljaeger.org/ - ckeil@vokaljaeger.org)")
writeInfoLine(	myHeader$ + "- version: 11/06/2020")
writeInfoLine(	myHeader$ + "- license: CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/)")
writeInfoLine(  myHeader$ + "")
writeInfoLine(  myHeader$ + "=== call parameters:")
writeInfoLine(  myHeader$ + "- sound directory    : ", myWavDirectory$)
writeInfoLine(  myHeader$ + "- sound file name    : ", myWavName$)
writeInfoLine(  myHeader$ + "- sound file extens. : ", myWavExtension$)
writeInfoLine(  myHeader$ + "- TextGrid directory : ", myTextGridDirectory$)
writeInfoLine(  myHeader$ + "- TextGrid name      : ", myTextGridName$)
writeInfoLine(  myHeader$ + "- result directory   : ", myOutputdirectory$)
writeInfoLine(  myHeader$ + "- result name        : ", tagOut$)
writeInfoLine(  myHeader$ + "- ...tail            : ", tailTagOut$)
writeInfoLine(  myHeader$ + "- channel to extract : ", theChannel )
writeInfoLine(  myHeader$ + "- phonem tier is     : ", shutterTier )
writeInfoLine(  myHeader$ + "- annotation tier is : ", annotationTier )
writeInfoLine(  myHeader$ + "- numeric offset     : ", offset)
writeInfoLine(  myHeader$ + "- lower frequency    : ", lowerFrequency, " Hz")
writeInfoLine(  myHeader$ + "- upper frequency    : ", upperFrequency, " Hz")
writeInfoLine(  myHeader$ + "- frequency stepwidth: ", frequencyStepWidth, " Hz")
writeInfoLine(  myHeader$ + "- window length      : ", windowLength, " msec")
writeInfoLine(  myHeader$ + "- window shift       : ", windowShift, " msec")

if ( maxCalcs == -1 ) 
	writeInfoLine(  myHeader$ + "- calculation limit  : none")
else
	writeInfoLine(  myHeader$ + "- calculation limit  : ", maxCalcs)
endif

writeInfoLine(  myHeader$ + "- filter set         : ", filterSet$)

writeInfoLine(  myHeader$ + "")
writeInfoLine(  myHeader$ + "=== Preparing        : ")
Text writing preferences: "UTF-8"

filepathWAV$ = myWavDirectory$ + "\" + myWavName$ + "." + myWavExtension$
writeInfoLine(  myHeader$ + "- going for sound    : ", filepathWAV$)
do ( "Read from file...", filepathWAV$ )

myChannel$ = myWavName$ + "_ch" + string$(theChannel)
writeInfoLine(  myHeader$ + "- going for channel  : ", myChannel$)
do ( "Extract one channel...", theChannel)

filepathTextGrid$ 	= myTextGridDirectory$ + "\" + 	myTextGridName$ 	+ ".TextGrid"
writeInfoLine(  myHeader$ + "- going for TextGrid : ", filepathTextGrid$ )
do ( "Read from file...", filepathTextGrid$ )
# IMPORTANT: select must  be immediatly followed by CR LF!
select TextGrid 'myTextGridName$'
noi = Get number of intervals... shutterTier							; phonems are stored in the tier [shutterTier], and we find [noi] of them
writeInfoLine(  myHeader$ + "- phonemes found     : ", noi)

myOutFile$	= myOutputdirectory$ + 	"\VJformantsRaw_" + tagOut$ 	+ "_" + tailTagOut$ + ".txt"
writeInfoLine(  myHeader$ + "- write results into : ", myOutFile$)

nl = (upperFrequency-lowerFrequency)/frequencyStepWidth+1				; number of limits to be tested
writeInfoLine(  myHeader$ + "- number of sweeps   : ", nl)

s$ = date$ ()
writeInfoLine(  myHeader$ + "")
writeInfoLine(  myHeader$ + "=== Starting calculation: ", s$)

	wl = windowLength / 1000											; window length in seconds
	ws = windowShift / 1000 											; window shift in seconds

	# --- write header line
	# --- naming: sample ~ phonem
	writeFileLine (myOutFile$,"tag",tab$,"tailtag",tab$,"tf",tab$,"annotationID",tab$,"annotation",tab$,"sampleID",tab$,"sample",tab$,"previoussample",tab$,"nextsample",tab$,"t",tab$,"tRel",tab$,"tRelNrm",tab$,"fii",tab$,"F1",tab$,"F2",tab$,"F3",tab$,"F4",tab$,"B1",tab$,"B2",tab$,"B3",tab$,"B4",tab$,"I0")

	# --- calculate intensity contour
	writeInfoLine(  myHeader$ + "calculating intensity...")
	s$ = date$ ()
	writeInfoLine(  myHeader$ + "...", s$)
	select Sound 'myChannel$'
	To Intensity: 75, ws
	writeInfoLine(  myHeader$ + "...done")

	# --- calculate formants
	writeInfoLine(  myHeader$ + "calculating formants...")

	# --- sweep frequencies
	tcf = 0
	for nfi from 1 to nl												; sweep all limits
		# --- tf = this frequency = actual upper limit we shall apply in praats formant tracker in a second
		if nl=1 
			tf = lowerFrequency
		else 
			tf = lowerFrequency + (nfi-1)*(upperFrequency-lowerFrequency)/(nl-1)		
		endif
		
		s$ = date$ ()
		writeInfoLine(  myHeader$ + "..." + s$ + ": applying upper frequency ", tf, " Hz. Remaining sweeps: ", nl-nfi+1, ".")
		select Sound 'myChannel$'
		To Formant (burg)... ws 5 tf wl 50								; we go in with fixed formant number, but sweep the upper frequency ceiling

		ci 	= 0															; reset count phonems calcs
		cf 	= 0															; reset count calculations
		ti 	= 1															; reset: thats the first annotation we shall test later... 
		
		select TextGrid 'myTextGridName$'
		
		previouslabel$ = "##"
		
		chunklabel$ = Get label of interval... shutterTier 1			; the 1st phonem 
		chunklabel$ = replace$(chunklabel$, " ", "", 100) 				; remove all spaces, which actually could leave us with nothing
		
		nextlabel$ = Get label of interval... shutterTier 2				; the 2nd phonem 
		nextlabel$ = replace$(chunklabel$, " ", "", 100) 				; remove all spaces, which actually could leave us with nothing
		
		i 	= 1
		while i <= noi and ( ci < maxCalcs or maxCalcs == -1)			; i ..noi runs through all phonems			
			select TextGrid 'myTextGridName$'
			
			if (i>1)													; NB: for i==1 we set already everything in variable initializatin
				chunklabel$ = nextlabel$								; the phonem at hand
				if (i==noi)
					nextlabel$ = "##"
				else
					nextlabel$ = Get label of interval... shutterTier (i+1)	; the next phonem 
					nextlabel$ = replace$(nextlabel$, " ", "", 100)
					nextlabelreport$ = nextlabel$
				endif
			endif
			
			# --- test on type of phonem
			tst$ 		= left$(chunklabel$,1)							; 1st character of phonem, apply filter set to that
			if tst$=="" 
				type = 0
			elsif tst$=="i" or tst$=="I" or tst$=="y" or tst$=="Y" or tst$=="u" or tst$=="U" or tst$=="E" or tst$=="6" or tst$=="@" or tst$=="e" or tst$=="o" or tst$=="O" or tst$=="2" or tst$=="9" or tst$=="a" or tst$=="A" or tst$=="{" or tst$=="&" or tst$=="Q" or tst$=="V" or tst$=="7" or tst$=="M" or tst$=="1" or tst$=="}" or tst$=="8" or tst$=="2" or tst$=="4" 
				# --- vowels
				type = 1 
			else
				# --- consonants (rest)
				type = 2
			endif
			
			# --- now check if we have found a non-null phonem marker AND it passes the filter
			if type>0 and (filterSet$ == "none" or (filterSet$ == "vowels" and type==1) or (filterSet$ == "consonants" and type==2))
				#writeInfoLine("processing phonem: ",chunklabel$)
				sp 	= Get starting point... shutterTier i				; phonem starts here
				ep 	= Get end point... shutterTier i					; phonem ends there
				dur = round((ep-sp)*1000)								; duration of phonem in ms

				# --- find annotation of phonem in annotation tier
				noii 		= Get number of intervals... annotationTier	; thats the annotation labels
				annotation$ = "_unknown_"
				cwl 		= -1 										; mark that we have not found no annotation for this phonem yet
				
				# --- resume search in annotation, where you left off the last time 		 
				while  cwl == -1 and ti <= noii
					spt 	= Get starting point... annotationTier ti
					ept 	= Get end point... annotationTier ti
					labelt$ = Get label of interval... annotationTier ti
					labelt$ = replace$(labelt$, " ", "", 100) 			; remove all spaces, which actually could leave us with nothing
					#writeInfoLine("testing label: ",labelt$)
					if cwl == -1 and labelt$ <> "" and sp >= spt and ep <= ept
						# --- we found annotation = is in range
						annotation$ 	= labelt$
						cwl 			= ti + offset
						# --- check if we keep previous and next label/phonem
						if (sp==spt)
							previouslabel$ = "##"						; previous phonem belongs to preceding annotation/word
						endif
						if (ep==ept) 
							nextlabelreport$ = "##"						; next phonem belongs to following annotation/word
						endif
					elsif spt >= ep 
						# --- terminate if we are past the point: the starting point of annotation is after the end point of the phonem = there is no annotation
						cwl = -2
						# --- and hence we dont know if we have true leading or tailing phonems, kill 
						previouslabel$ = "#"
						nextlabelreport$ = "#"
					else 		
						# --- go on searching
						ti 	= ti + 1
					endif
				endwhile	
				#writeInfoLine("found annotation: ",annotation$)

				# --- now shift through the phonem an extract actual formant trajectory
				nf = (ep-sp)/ws											; number of ws-shifted windows in phomen
				
				for fii from 0 to nf									; sweep ws-shifted windows
					trel = fii * ws										; intra phonem relative time [msec]
					trelnrm = (fii * ws)/(ep-sp)*100					; intra phonem relative time [percentage]
					t = sp + trel
										
					# --- get formant shape in window
					select Formant 'myChannel$'
					
					f1 = Get value at time... 1 't' Hertz linear
					f2 = Get value at time... 2 't' Hertz linear
					f3 = Get value at time... 3 't' Hertz linear
					f4 = Get value at time... 4 't' Hertz linear

					b1 = Get bandwidth at time... 1 't' Hertz linear
					b2 = Get bandwidth at time... 2 't' Hertz linear
					b3 = Get bandwidth at time... 3 't' Hertz linear
					b4 = Get bandwidth at time... 4 't' Hertz linear
			
					# --- get intensity contour
					select Intensity 'myChannel$'
					intensity = Get value at time: 't', "Cubic"
					
					# --- report 
					appendFileLine (myOutFile$,tagOut$,tab$,tailTagOut$,tab$,tf,tab$,cwl,tab$,annotation$,tab$,(offset+i),tab$,chunklabel$,tab$,previouslabel$,tab$,nextlabelreport$,tab$,t,tab$,trel,tab$,trelnrm,tab$,fii,tab$,f1,tab$,f2,tab$,f3,tab$,f4,tab$,b1,tab$,b2,tab$,b3,tab$,b4,tab$,intensity)						
					cf = cf +1
					tcf = tcf +1
				endfor													; ws-shift sweep intra phonem
				ci = ci +1												; increase calculated phonem counter
			endif
			i = i +1 													; increase phonem number
			previouslabel$ = chunklabel$
		endwhile														; all phonems

		# --- clean up
		select Formant 'myChannel$'
		Remove
	endfor																; upper limit frequency sweep

	#  --- clean up
	select Intensity 'myChannel$'
	Remove
	writeInfoLine(  myHeader$ + "...done")
	
s$ = date$ ()
writeInfoLine(  myHeader$ + "")
writeInfoLine(  myHeader$ + "=== Ended: ", s$)
writeInfoLine(  myHeader$ + "- phonems found          : ", i-1)
writeInfoLine(  myHeader$ + "- ...thereof processed   : ", ci)
writeInfoLine(  myHeader$ + "- calculations per sweep : ", cf)
writeInfoLine(  myHeader$ + "- total calculations     : ", tcf)
writeInfoLine(  myHeader$ + "=== ===> Closing " + tagOut$ + " (" + tailTagOut$ + ")." )