#!/usr/bin/env Rscript

# --- check if we are interactive or in true script mode
if (!interactive()) {
  args <- commandArgs(trailingOnly=TRUE)
  # 1: config file name
} else {
  setwd("C:\\Users\\User\\Desktop\\VJ.EAT.demo\\do_jobs")
  #setwd("c:\\Users\\User\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT.demo\\do_jobs") # PC
  #setwd("c:\\Users\\User\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT.Tagesschau\\do_jobs") 
  #setwd("c:\\Users\\ckeil\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT.demo\\do_jobs") # laptop
  #args <- c('..\\job_config\\5 VJconfig_VJplotFormants_long_monophthongs.txt','..\\job_config\\VJconfig_job_meta.txt')
  args <- c('C:\\Users\\User\\Desktop\\VJ.EAT.demo\\job_config\\5 VJconfig_VJanalyzeFormants_monophthong_stats.txt','..\\job_config\\VJconfig_job_meta.txt')
  args <- c('C:\\Users\\User\\Desktop\\VJ.EAT.demo\\job_config\\5 VJconfig_VJanalyzeFormants_long_u_spectrum.txt','..\\job_config\\VJconfig_job_meta.txt')
  #args <- c('..\\job_config\\5 VJconfig_VJplotFormants_long_u_spectrum.txt','..\\job_config\\VJconfig_job_meta.txt')
  #args <- c('..\\job_config\\5 VJconfig_VJplotFormants_short_monophthongs.txt','..\\job_config\\VJconfig_job_meta.txt')
  
  #setwd("c:\\Users\\User\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT.Kielkorpus\\do_jobs") # PC 
  #args <- c('..\\job_config\\5 VJconfig_VJplotFormants_long_monophthongs-KielKorpus.txt','..\\job_config\\VJconfig_job_meta-KielKorpus-sel.txt')
}

# --- start up
print("=== VokalJaeger Enhanced Algorithmic Tool pack (VJ.EAT): VJanalyzeFormants.R")
print("- author : Carsten Keil (http://vokaljaeger.org/ - ckeil@vokaljaeger.org)")
print("- this program was last updated with VJ.EAT version 0.15.")
print("- license: CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/)")
print("- call parameters:")
print(args)
print('')

# --- get libraries
print("=== Starting R Kernel")
# --- R standards
print(R.version.string)
print("--- Loading R-libraries from:")
print(.libPaths())
library(data.table)
library(ggplot2)
library(hqmisc)
library(MASS)
library(ICSNP)

# --- read config file and store in list
print("=== Getting configurations")
myCnfg <- list(inCnfgFle=args[1])
myCnfg$BseDrctry <- getwd()
print(paste0('Going for configuration file >',myCnfg$inCnfgFle,'<'))
tmp <- fread(file=myCnfg$inCnfgFle, header=FALSE)
myInCnfg <- lapply(setNames(split(tmp$V2, seq(nrow(tmp))), tmp$V1), FUN=function(x) as.character(x))

# --- set/get VJ base directory and load libs from there
if( with(myInCnfg, exists('VJDirectory'))) {
  myCnfg$VJDrctry <- myInCnfg$VJDirectory
} else {
  myCnfg$VJDrctry <- getwd()
}
print(paste0(" - Now getting VJ libs from >", myCnfg$VJDrctry,"<..."))
setwd(myCnfg$VJDrctry)
source("lib\\R\\VJutils.R")
source("lib\\R\\VJanalyzeKernel.R")
print('')


myCnfg$OutDrctry <- VJgetProtectedConfig(myInCnfg, "outDirectory", default='.')
print(paste0(" - outbound directory >", myCnfg$OutDrctry,"<."))

myCnfg$PltTtle <- VJgetProtectedConfig(myInCnfg, "plotTitle", default='')

myCnfg$PltIncldRgx <- VJgetProtectedConfig(myInCnfg, "plotRegexIncludeSampleContext", default='#all')
myCnfg$PltExcldRgx <- VJgetProtectedConfig(myInCnfg, "plotRegexExcludeSampleContext", default='#none')
myCnfg$PltIncldeTg <- VJgetProtectedConfig(myInCnfg, "plotIncludeTag", default='#all')

myCnfg$PltOrgnl <- VJgetProtectedConfig(myInCnfg, "plotOriginal", default='yes')
myCnfg$PltNrmlzed <- VJgetProtectedConfig(myInCnfg, "plotNormalized", default='yes')
myCnfg$PltSpctrm <- VJgetProtectedConfig(myInCnfg, "plotSpectrum", default='no')
myCnfg$PltStstcs <- VJgetProtectedConfig(myInCnfg, "plotStatistics", default='no')
myCnfg$EllpseInclsnRte <- VJgetProtectedConfig(myInCnfg, "plotEllipseinclusionRate", default=0.5, forceNumeric = TRUE)

myCnfg$PltTrngle <- VJgetProtectedConfig(myInCnfg, "plotTriangle", default='none')


print(myCnfg)

# === process steering file
setwd(myCnfg$BseDrctry)
myCnfg$StrngFle <- args[2]
print(paste0("--- Processing job meta steering file >",myCnfg$StrngFle, "< and found:"))
myStrngFle <- fread(file=myCnfg$StrngFle, sep="\t")
print(myStrngFle)
print('')

# === get in formants
myCnfg$InDrctry <- VJgetProtectedConfig(myInCnfg, "inDirectory", default='.')
print(paste0("--- Processing formant file for normalization from directory >", myCnfg$InDrctry,"<:"))
setwd(myCnfg$InDrctry)

myFrmnts2Plt <- do.call(rbind.data.frame,
                        lapply(myStrngFle[doUse=='Y' & doPlot=='Y',]$tag, FUN=function(tag) {
                          # --- loop inbound formant files
                          fn <- paste0('VJformantsNormalized ',tag,'.txt')
                          print(paste0("...going for formant file >",fn,"<..."))
                          tmp <- fread(file=fn, sep="\t", quote="", header=TRUE, dec='.')
                          print(paste0("... ...found ", VJtidyNumber(nrow(tmp)), " lines of data."))
                          return(tmp)
                        }))

print(paste0("Found in total ", VJtidyNumber(nrow(myFrmnts2Plt)), " lines of data in ", myStrngFle[doUse=='Y' & doPlot=='Y',.N], " files to analyze."))
print(paste0("..breakdown totals by tag:"))
print(myFrmnts2Plt[,.(lines=VJtidyNumber(.N)),by=tag])
print('')

# --- handle specific in- and exclusions to define the reference universe
if (myCnfg$PltIncldRgx == '#all') { 
  myFrmnts2Plt$Ninclde_ <- TRUE  
} else  {
  myFrmnts2Plt$Ninclde_ <- grepl(myCnfg$PltIncldRgx, myFrmnts2Plt$samplecontext, perl = TRUE) 
}
if (myCnfg$PltExcldRgx == '#none') { 
  myFrmnts2Plt$Nexclde_ <- FALSE  
} else  {
  myFrmnts2Plt$Nexclde_ <- grepl(myCnfg$PltExcldRgx, myFrmnts2Plt$samplecontext, perl = TRUE) 
}
myFrmnts2Plt$PltIndx <- with(myFrmnts2Plt, Ninclde_ & !Nexclde_)

# check if we are to restrict on a selected tag
if (myCnfg$PltIncldeTg != '#all') {myFrmnts2Plt[, PltIndx := PltIndx & (tag==myCnfg$PltIncldeTg)]}

print('Breakdown of records (not) to be plotted')
dcast(myFrmnts2Plt[, .N, by=.(tag,PltIndx)], PltIndx~tag, value.var = 'N') 

print('Plotting the following samples')
if (myFrmnts2Plt[(PltIndx),.N]) { dcast(myFrmnts2Plt[(PltIndx), .N, by=.(tag,sample)], sample~tag, value.var = 'N') } # V0.15: protect for empty data set

# === get VJ tab 02 = vowel order
setwd(myCnfg$VJDrctry)
myVwlOrdr <- fread(file="data\\VJtab02.txt")
myVwlOrdr[, V := factor(V, levels=V)]
myVwlOrdr[, L := 0] # create long/short marker, here: short
myVwlOrdr <- rbind(myVwlOrdr, myVwlOrdr[, .(V,O,P,R,L=1)]) # create artifical long versions and attach
myVwlOrdr[, VV := paste0(V,ifelse(L==0,'',':'))] # create vowel sign including long-marker
myVwlOrdr[, VV := factor(VV, levels=myVwlOrdr[order(V,L)]$VV)]
myVwlOrdr[, OO := factor(O*4 + (1-L) - 3, levels=14:1, labels=c('opn.shrt','opn.long','lessOpn.shrt','lessOpn.long','hlfOpn.shrt','hlfOpn.long','midOpn.shrt', 'midOpn.long', 'hlfClsd.shrt','hlfClsd.long','lessClsd.shrt','lessClsd.long','clsd.shrt','clsd.long'))] # ordinal openess for plotting
myVwlOrdr[, PP := factor(P*2 + R - 1, levels=1:6, labels=c('frnt.nRnd','frnt.Rnd','cntr.nRnd','cntR.Rrn','back.nRnd','back.Rnd'))] # ordinal position for plotting

# myVwlOrdr[order(OO, PP)]

setwd(myCnfg$BseDrctry)
setwd(myCnfg$OutDrctry)

if (myFrmnts2Plt[(PltIndx),.N] == 0) {
  warning('Nothing to plot?')
} else {
  if (myCnfg$PltOrgnl=='yes') {
    print("Plotting original values...")
    myFrmnts2Plt[, c('myF1b','myF2b','myF3b') := lapply(.SD, f2bark), .SDcols=c('F1P','F2P','F3P')] # everything runs of bark, so prepare
    with(myCnfg, VJplotFormantTriangleGrid( myFrmnts2Plt, myTitle=PltTtle, myTitleTail='All--Orgnl Pt', plotTriangle = PltTrngle))
    with(myCnfg, VJplotFormantTriangleGrid( myFrmnts2Plt, myTitle=PltTtle, myTitleTail='All--Orgnl PtEl', plotEllipse = TRUE, plotPoints = TRUE, EllipseinclusionRate = EllpseInclsnRte))
    with(myCnfg,  VJplotFormantTriangleGrid( myFrmnts2Plt, myTitle=PltTtle, myTitleTail='All--Orgnl El', plotEllipse = TRUE, plotPoints = FALSE, EllipseinclusionRate = EllpseInclsnRte))
    
    lapply(split(myFrmnts2Plt, myFrmnts2Plt$tag), function(x)  {
      with(myCnfg, VJplotFormantTriangleGrid( x, myTitle=PltTtle, myTitleTail=paste0(x[1,]$tag,'--Orgnl Pt')))
      with(myCnfg, VJplotFormantTriangleGrid( x, myTitle=PltTtle, myTitleTail=paste0(x[1,]$tag,'--Orgnl PtEl'), plotEllipse = TRUE, plotPoints = TRUE, EllipseinclusionRate = EllpseInclsnRte))
      with(myCnfg, VJplotFormantTriangleGrid( x, myTitle=PltTtle, myTitleTail=paste0(x[1,]$tag,'--Orgnl El'), plotEllipse = TRUE, plotPoints = FALSE, EllipseinclusionRate = EllpseInclsnRte))
    })
  }
  
  if (myCnfg$PltNrmlzed=='yes') {
    myFrmnts2Plt[, c('myF1b','myF2b','myF3b') := lapply(.SD, f2bark), .SDcols=c('F1PN','F2PN','F3PN')] # everything runs of bark, so prepare
    print("Plotting normalized values...")
    with(myCnfg, VJplotFormantTriangleGrid( myFrmnts2Plt, myTitle=PltTtle, myTitleTail='All--Nrmlzd Pt'))
    with(myCnfg, VJplotFormantTriangleGrid( myFrmnts2Plt, myTitle=PltTtle, myTitleTail='All--Nrmlzd PtEL', plotEllipse = TRUE, plotPoints = TRUE, EllipseinclusionRate = EllpseInclsnRte))
    with(myCnfg, VJplotFormantTriangleGrid( myFrmnts2Plt, myTitle=PltTtle, myTitleTail='All--Nrmlzd El',plotEllipse = TRUE, plotPoints = FALSE, EllipseinclusionRate = EllpseInclsnRte))
    
    lapply(split(myFrmnts2Plt, myFrmnts2Plt$tag), function(x)  {
      with(myCnfg, VJplotFormantTriangleGrid( x, myTitle=PltTtle, myTitleTail=paste0(x[1,]$tag,'--Nrmlzd Pt')))
      with(myCnfg, VJplotFormantTriangleGrid( x, myTitle=PltTtle, myTitleTail=paste0(x[1,]$tag,'--Nrmlzd PtEl'), plotEllipse = TRUE, plotPoints = TRUE, EllipseinclusionRate = EllpseInclsnRte))
      with(myCnfg, VJplotFormantTriangleGrid( x, myTitle=PltTtle, myTitleTail=paste0(x[1,]$tag,'--Nrmlzd El'), plotEllipse = TRUE, plotPoints = FALSE, EllipseinclusionRate = EllpseInclsnRte))
    })
  }
  
  if (myCnfg$PltSpctrm=='yes') {
    tmp <- myFrmnts2Plt[(PltIndx), VJplotSpectrum( cbind(.SD, sampleID=.BY)), by=sampleID] # the [tmp] catches some odd data.table warning ???
  }
  
  if (myCnfg$PltStstcs=='yes') {
    print("Calculating statistic on original values...")
    myFrmnts2Plt[, c('myF1b','myF2b','myF3b') := lapply(.SD, f2bark), .SDcols=c('F1P','F2P','F3P')] # everything runs of bark, so prepare
    lapply(split(myFrmnts2Plt, myFrmnts2Plt$tag), function(x)  {VJplotStatistics( x,  myTitle=myCnfg$PltTtle, myTitleTail=paste0(x[1,]$tag,'--Orgnl'))})

    print("...Hotelling T2")
    lapply(split(myFrmnts2Plt, myFrmnts2Plt$tag), function(x)  {VJanalyzeDifference( x,  myTitle=myCnfg$PltTtle, myTitleTail=paste0(x[1,]$tag,'--Orgnl'), debugLevel=0)})
    
    print("Calculating statistic on normalized  values...")
    myFrmnts2Plt[, c('myF1b','myF2b','myF3b') := lapply(.SD, f2bark), .SDcols=c('F1PN','F2PN','F3PN')] # everything runs of bark, so prepare
    lapply(split(myFrmnts2Plt, myFrmnts2Plt$tag), function(x)  {VJplotStatistics( x,  myTitle=myCnfg$PltTtle, myTitleTail=paste0(x[1,]$tag,'--Nrmlzd'))})
  
  
  }
}

