#!/usr/bin/env Rscript

# --- check if we are interactive or in true script mode
if (!interactive()) {
  args <- commandArgs(trailingOnly=TRUE)
  # 1: config file name
} else {
  #setwd("c:\\Users\\User\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT\\demo\\do_jobs") # PC
  setwd("c:\\Users\\ckeil\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT\\demo\\do_jobs") # laptop
  args <- c('..\\job_config\\4 VJconfig_VJclassifyFormants.txt','..\\job_config\\VJconfig_job_meta.txt')
}

# --- start up
print("=== VokalJaeger Enhanced Algorithmic Toolpack: VJclassifyFormants.R")
print("- author : Carsten Keil (http://vokaljaeger.org/ - ckeil@vokaljaeger.org)")
print("- version: xxx/05/2020")
print("- license: CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/)")
print("- call parameters:")
print(args)
print('')

# --- get libraries
print("=== Starting R Kernel")
# --- R standards
print(R.version.string)
print("--- Loading R-libraries from:")
print(.libPaths())
library(data.table)

# --- VJ stuff
source("..\\..\\lib\\R\\VJutils.R")
print('')

# --- read config file and store in list
print("=== Getting configurations")
myCnfg <- list(inCnfgFle=args[1])
myCnfg$BseDrctry <- getwd()
print(paste0(" - reading config file >", myCnfg$inCnfgFle,"<."))
tmp <- fread(args[1], header=FALSE)
myInCnfg <- lapply(setNames(split(tmp$V2, seq(nrow(tmp))), tmp$V1), FUN=function(x) as.character(x))

myCnfg$OutDrctry <- VJgetProtectedConfig(myInCnfg, "outDirectory", default='.')
print(paste0(" - outbound directory >", myCnfg$OutDrctry,"<."))


# === process steering file
myCnfg$StrngFle <- args[2]
print(paste0("--- Processing job meta steering file >",myCnfg$StrngFle, "< and found:"))
myStrngFle <- fread(myCnfg$StrngFle, sep="\t")
print(myStrngFle)
print('')

# === get in formants
myCnfg$InDrctry <- VJgetProtectedConfig(myInCnfg, "inDirectory", default='.')
print(paste0("--- Processing formant file for normalization from directory >", myCnfg$InDrctry,"<:"))
setwd(myCnfg$InDrctry)

myFrmnts2Clssfy <- do.call(rbind.data.frame,
                        lapply(myStrngFle[doUse=='Y' & doClassify=='Y',]$tag, FUN=function(tag) {
                          # --- loop inbound formant files
                          fn <- paste0('VJformantsNormalized ',tag,'.txt')
                          print(paste0("...going for formant file >",fn,"<..."))
                          tmp <- fread(file=fn, sep="\t", quote="", header=TRUE, dec='.')
                          print(paste0("... ...found ", VJtidyNumber(nrow(tmp)), " lines of data."))
                          return(tmp)
                        }))

print(paste0("Found in total ", VJtidyNumber(nrow(myFrmnts2Clssfy)), " lines of data in ", myStrngFle[doUse=='Y' & doPlot=='Y',.N], " files for classification"))
print(paste0("..breakdown totals by tag:"))
print(myFrmnts2Clssfy[,.(lines=VJtidyNumber(.N)),by=tag])
print('')

