#!/usr/bin/env Rscript

# --- check if we are interactive or in true script mode
if (!interactive()) {
  args <- commandArgs(trailingOnly=TRUE)
  # 1: config file name
} else {
  #setwd("c:\\Users\\User\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT\\demo\\do_jobs") # PC
  #setwd("c:\\Users\\ckeil\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT\\demo\\do_jobs") # laptop
  #args <- c('..\\job_config\\3 VJconfig_VJnormalizeFormants.txt','..\\job_config\\VJconfig_job_meta.txt')
  
  setwd("C:\\Users\\User\\Desktop\\VJ.EAT.Roland\\do_jobs") # PC 
  args <- c('..\\job_config\\3 VJconfig_VJnormalizeFormants.txt','..\\job_config\\VJconfig_job_meta.txt')
}

# --- start up
print("=== VokalJaeger Enhanced Algorithmic Toolpack: VJnormalizeFormants.R")
print("- author : Carsten Keil (http://vokaljaeger.org/ - ckeil@vokaljaeger.org)")
print("- this program was last updated with version 0.12.")
print("- license: CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/)")
print("- call parameters:")
print(args)
print('')

# --- get libraries
print("=== Starting R Kernel")
# --- R standards
print(R.version.string)
print("--- Loading R-libraries from:")
print(.libPaths())
library(data.table)
library(hqmisc)

# --- read config file and store in list
print("=== Getting configurations")
myCnfg <- list(inCnfgFle=args[1])
myCnfg$BseDrctry <- getwd()
print(paste0('Going for configuration file >',myCnfg$inCnfgFle,'<'))
tmp <- fread(file=myCnfg$inCnfgFle, header=FALSE)
myInCnfg <- lapply(setNames(split(tmp$V2, seq(nrow(tmp))), tmp$V1), FUN=function(x) as.character(x))

# --- set/get VJ base directory and load libs from there
if( with(myInCnfg, exists('VJDirectory'))) {
  myCnfg$VJDrctry <- myInCnfg$VJDirectory
} else {
  myCnfg$VJDrctry <- getwd()
}
print(paste0(" - Now getting VJ libs from >", myCnfg$VJDrctry,"<..."))
setwd(myCnfg$VJDrctry)
source("lib\\R\\VJutils.R")
source("lib\\R\\VJnormalizeKernel.R")
print('')

myCnfg$OutDrctry <- VJgetProtectedConfig(myInCnfg, "outDirectory", default='.')
print(paste0(" - outbound directory >", myCnfg$OutDrctry,"<."))

myCnfg$F1hntng <- VJgetProtectedConfig(myInCnfg, "normalizeF1hinting", default='yes')
myCnfg$NrmlztnMthdFrmnts <- VJgetProtectedConfig(myInCnfg, "normalizeMethodFormants", default='robustLobanov')
myCnfg$NrmlztnMthdBndWdth <- VJgetProtectedConfig(myInCnfg, "normalizeMethodBandwidh", default='robustZ')
myCnfg$NrmlztnQntleBndStrt <- VJgetProtectedConfig(myInCnfg, "normalizeBandStart", default=0.05, forceNumeric = TRUE)
myCnfg$NrmlztnQntleBndWdth <- VJgetProtectedConfig(myInCnfg, "normalizeBandWidth", default=0.04, forceNumeric = TRUE)
myCnfg$NrmlztnIncldRgx <- VJgetProtectedConfig(myInCnfg, "normalizeRegexIncludeSampleContext", default='#all')
myCnfg$NrmlztnExcldRgx <- VJgetProtectedConfig(myInCnfg, "normalizeRegexExcludeSampleContext", default='#none')
myCnfg$NrmlztnMxErr <- VJgetProtectedConfig(myInCnfg, "normalizeMaxError", default=0.9, forceNumeric = TRUE)

myCnfg$NrmlztnDoBckPrjctn <- VJgetProtectedConfig(myInCnfg, "normalizeBackprojectFormants", default='yes')
if (with(myCnfg, NrmlztnMthdFrmnts!='robustLobanov' & NrmlztnDoBckPrjctn=='yes')) {
  print("WARNING: Backprojection works only for robust Lobanov. Now disabled!")
  myCnfg$NrmlztnDoBckPrjctn <- 'no'
}

# === process steering file
setwd(myCnfg$BseDrctry)
myCnfg$StrngFle <- args[2]
print(paste0("--- Processing job meta steering file >",myCnfg$StrngFle, "< and found:"))
myStrngFle <- fread(file=myCnfg$StrngFle, sep="\t")
print(myStrngFle)
print('')

# === get in formants
myCnfg$InDrctry <- VJgetProtectedConfig(myInCnfg, "inDirectory", default='.')
print(paste0("--- Processing formant file for normalization from directory >", myCnfg$InDrctry,"<:"))
setwd(myCnfg$InDrctry)

myFrmnts2Nrmlze <- do.call(rbind.data.frame,
                      lapply(myStrngFle[doUse=='Y' & doNormalize=='Y',]$tag, FUN=function(tag) {
                        # --- loop inbound formant files
                        fn <- paste0('VJformantsSwept ',tag,'.txt')
                        print(paste0("...going for formant file >",fn,"<..."))
                        tmp <- fread(file=fn, sep="\t", quote="", header=TRUE, dec='.')
                        print(paste0("... ...found ", VJtidyNumber(nrow(tmp)), " lines of data."))
                        return(tmp)
                      }))
                      
print(paste0("Found in total ", VJtidyNumber(nrow(myFrmnts2Nrmlze)), " lines of data in ", myStrngFle[doUse=='Y' & doNormalize=='Y',.N], " files for normalization."))
print(paste0("..breakdown totals by tag:"))
print(myFrmnts2Nrmlze[,.(lines=VJtidyNumber(.N)),by=tag])
print('')

# --- handle specific in- and exclusions to define the reference universe
if (myCnfg$NrmlztnIncldRgx == '#all') { 
  myFrmnts2Nrmlze$Ninclde_ <- TRUE  
} else  {
  myFrmnts2Nrmlze$Ninclde_ <- grepl(myCnfg$NrmlztnIncldRgx, myFrmnts2Nrmlze$samplecontext, perl = TRUE) 
}
if (myCnfg$NrmlztnExcldRgx == '#none') { 
  myFrmnts2Nrmlze$Nexclde_ <- FALSE  
} else  {
  myFrmnts2Nrmlze$Nexclde_ <- grepl(myCnfg$NrmlztnExcldRgx, myFrmnts2Nrmlze$samplecontext, perl = TRUE) 
}
myFrmnts2Nrmlze[, NrmlztnRfrnce :=  Ninclde_ & !Nexclde_ & FltErr<0]
#myFrmnts2Nrmlze[, .N, NrmlztnRfrnce]

tmp.stats <- myFrmnts2Nrmlze[(NrmlztnRfrnce), .(ErrThrshld=quantile(FltErr, myCnfg$NrmlztnMxErr)), by=tag]
myFrmnts2Nrmlze <- merge(myFrmnts2Nrmlze, tmp.stats, by='tag')
myFrmnts2Nrmlze[, NrmlztnRfrnce :=  NrmlztnRfrnce & FltErr<ErrThrshld]

print('Breakdown of records (not) qualifying as normalization reference')
dcast(myFrmnts2Nrmlze[, .N, by=.(tag,NrmlztnRfrnce)], NrmlztnRfrnce~tag, value.var = 'N') 

print('Using the following record as normalization reference')
if (myFrmnts2Nrmlze[(NrmlztnRfrnce), .N]>0) {
  dcast(myFrmnts2Nrmlze[(NrmlztnRfrnce), .N, by=.(tag,sample)], sample~tag, value.var = 'N')
} else {
  print('FATAL ERROR: nothing selected/found as normalization reference')
  quit()
}
print('Excluding the following records from the normalization refrence')
if (myFrmnts2Nrmlze[!(NrmlztnRfrnce), .N]>0) {
  dcast(myFrmnts2Nrmlze[!(NrmlztnRfrnce), .N, by=.(tag,sample)], sample~tag, value.var = 'N') 
} else {
  print('Nothing is excluded')
}
# === actual normalization

myPrmtrs <- with(myCnfg, list(
  convertToBark=TRUE,
  doBackProjection=NrmlztnDoBckPrjctn ,
  BandWidth=NrmlztnQntleBndWdth,
  BandStart=NrmlztnQntleBndStrt,
  MaxFilterError=NrmlztnMxErr
))

doClmns <- grep('F[123][LRP]$', colnames(myFrmnts2Nrmlze), value=TRUE)
myFrmntsNrmlzd <- myFrmnts2Nrmlze[, do.call( VJnormalizeDoSeries, c(list(.SD, AllSrs2Nrmlze=doClmns, method=myCnfg$NrmlztnMthdFrmnts), myPrmtrs)), by=tag]
myStstcs <- myFrmnts2Nrmlze[, do.call( VJnormalizeGetStatistics, c(list(.SD, AllSrs2Nrmlze=doClmns, method=myCnfg$NrmlztnMthdFrmnts), myPrmtrs)), by=tag]

doClmns <- grep('B[123]P', colnames(myFrmnts2Nrmlze), value=TRUE)
myFrmntsNrmlzd <- myFrmntsNrmlzd[, do.call( VJnormalizeDoSeries, c(list(.SD, AllSrs2Nrmlze=doClmns, method=myCnfg$NrmlztnMthdBndWdth), myPrmtrs)), by=tag]
myStstcs <- rbind(myStstcs, myFrmnts2Nrmlze[, do.call( VJnormalizeGetStatistics, c(list(.SD, AllSrs2Nrmlze=doClmns, method=myCnfg$NrmlztnMthdBndWdth), myPrmtrs)),  by=tag])

# === save
print("Saving formants and statistics")
myFrmntsNrmlzd[,VJmyNamedSave(.SD, tag=.BY, lead='VJformantsNormalized', directory=myCnfg$OutDrctry),by=tag]
myStstcs[,VJmyNamedSave(.SD, tag=.BY, lead='VJformantsNormalizedStatistics', directory=myCnfg$OutDrctry),by=tag]

