# --- check if we are interactive or in true script mode
if (!interactive()) {
  args <- commandArgs(trailingOnly=TRUE)
  # 1: config file name
} else {
  #setwd("c:\\Users\\User\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT.demo\\do_jobs") # PC
  #setwd("c:\\Users\\ckeil\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT.demo\\do_jobs") # laptop
  #args <- c('..\\job_config\\2 VJconfig_VJsweepFormants.txt','..\\job_config\\VJconfig_job_meta-KielKorpus-sel.txt')
  
  setwd("c:\\Users\\User\\OneDrive\\meinewelt\\1009 VokalJaeger\\VJ.EAT.Kielkorpus\\do_jobs") # PC 
  args <- c('..\\job_config\\2 VJconfig_VJsweepFormants-KielKorpus.txt','..\\job_config\\VJconfig_job_meta-KielKorpus-sel.txt')
}

# --- start up
print("=== VokalJaeger Enhanced Algorithmic Toolpack: VJsweepFormants.R")
print("- author : Carsten Keil (http://vokaljaeger.org/ - ckeil@vokaljaeger.org)")
print("- this program was last updated with version 0.10.")
print("- license: CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/)")
print("- call parameters:")
print(args)
print('')

# --- get libraries
print("=== Starting R Kernel")
# --- R standards
print(R.version.string)
print("--- Loading R-libraries from:")
print(.libPaths())
library(data.table)
library(doBy)
library(dtt)
library(ggplot2)
library(hqmisc)

# --- read config file and store in list
print("=== Getting configurations")
myCnfg <- list(inCnfgFle=args[1])
myCnfg$BseDrctry <- getwd()
tmp <- fread(file=args[1], header=FALSE)
myInCnfg <- lapply(setNames(split(tmp$V2, seq(nrow(tmp))), tmp$V1), FUN=function(x) as.character(x))

# --- set/get VJ base directory and load libs from there
if( with(myInCnfg, exists('VJDirectory'))) {
  myCnfg$VJDrctry <- myInCnfg$VJDirectory
} else {
  myCnfg$VJDrctry <- getwd()
}
print(paste0(" - Now getting VJ libs from >", myCnfg$VJDrctry,"<..."))
setwd(myCnfg$VJDrctry)
source("lib\\R\\VJutils.R")
source("lib\\R\\VJfoldBack.R")
source("lib\\R\\VJsweepKernel.R")
print('')

# --- report on what configuration you found (if not found, apply robust defaults)
myCnfg$OutDrctry <- VJgetProtectedConfig(myInCnfg, "outDirectory", default='.')
print(paste0(" - outbound directory >", myCnfg$OutDrctry,"<."))

myCnfg$RprtMde <- VJgetProtectedConfig(myInCnfg, "runMode", default='process')
print(paste0(" - report mode >", myCnfg$RprtMde,"<."))

myCnfg$RprtDrctry <- VJgetProtectedConfig(myInCnfg, "reportDirectory", default='.')
print(paste0(" - report directory >", myCnfg$RprtDrctry,"<."))



tmp <- unlist(strsplit(VJgetProtectedConfig(myInCnfg, "reportSample", default=-1),','))
myCnfg$RprtSmpleIDs <- as.numeric(tmp)
#myCnfg$RprtSmpleIDStrt <- as.numeric(tmp[1])
#myCnfg$RprtSmpleIDEnd <- myCnfg$RprtSmpleIDStrt
#if (length(tmp)>1 && myCnfg$RprtSmpleIDStrt < as.numeric(tmp[2])) {  myCnfg$RprtSmpleIDEnd <- as.numeric(tmp[2]) }

myCnfg$RgxSmple <- VJgetProtectedConfig(myInCnfg, "regexSample", default='.+')

myCnfg$PrlllDo <- VJgetProtectedConfig(myInCnfg, "sweepDoParallel", default='no')
myCnfg$PrlllJbs <- VJgetProtectedConfig(myInCnfg, "sweepNumberOfCores", default=1, forceNumeric = TRUE)
myCnfg$SwpCshn <- VJgetProtectedConfig(myInCnfg, "sweepCushion", default=0.2, forceNumeric = TRUE)
myCnfg$OrdrDTT <- VJgetProtectedConfig(myInCnfg, "sweepOrderDTT", default=3, forceNumeric = TRUE)
myCnfg$wgtBndWdth <- VJgetProtectedConfig(myInCnfg, "sweepWeightBandwith", default=0, forceNumeric = TRUE)
myCnfg$PckPntSngle <- VJgetProtectedConfig(myInCnfg, "sweepPickSingle", default=0.3, forceNumeric = TRUE)
myCnfg$PckPntPr <- VJgetProtectedConfig(myInCnfg, "sweepPickPair", default=0.2, forceNumeric = TRUE)
myCnfg$TrngleTst <- VJgetProtectedConfig(myInCnfg, "sweepApplyTriangleTest", default='no')
myCnfg$TrngleCrp <- VJgetProtectedConfig(myInCnfg, "sweepTriangleCreep", default=0.25, forceNumeric = TRUE)
myCnfg$AvrgngBnd <- VJgetProtectedConfig(myInCnfg, "sweepAveragingBand", default=0.02, forceNumeric = TRUE)
myCnfg$GndrHntMls <- VJgetProtectedConfig(myInCnfg, "sweepGenderHintMalus", default=0.2, forceNumeric = TRUE)
myCnfg$F3errWgt <- VJgetProtectedConfig(myInCnfg, "sweepF3errorWeight", default=1, forceNumeric = TRUE)
myCnfg$LctnPrbbltyWght <- VJgetProtectedConfig(myInCnfg, "sweepLocationProbabilityWeight", default=0.5, forceNumeric = TRUE)
myCnfg$LctnPrbbltyWdth <- VJgetProtectedConfig(myInCnfg, "sweepLocationProbabilityWidth", default=0.5, forceNumeric = TRUE)
myCnfg$mdloCndtn <- VJgetProtectedConfig(myInCnfg, "moduloCondition", default=-1, forceNumeric = TRUE)
print(myCnfg)

print('')

# === process steering file
setwd(myCnfg$BseDrctry)
myCnfg$StrngFle <- args[2]
print(paste0("--- Processing job meta steering file >",myCnfg$StrngFle, "< and found:"))
myStrngFle <- fread(myCnfg$StrngFle, sep="\t")
myStrngFle$GndrUse <- apply(myStrngFle[,'Gender'], 1, function(x) with(myCnfg, ifelse(TrngleTst=='yes' | GndrHntMls>0, x, 'none')))
print(myStrngFle)
print('')

# === get in raw formants
myCnfg$InDrctry <- VJgetProtectedConfig(myInCnfg, "inDirectory", default='.')
print(paste0("--- Processing raw formant file from directory >", myCnfg$InDrctry,"<:"))
setwd(myCnfg$InDrctry)
myRwFiles <- data.table(fleNme=list.files(pattern="VJformantsRaw.+\\.txt"))
myRwFiles$tag <- sapply(myRwFiles$fleNme, FUN=function(x) substr(x, start = nchar("VJformantsRaw_")+1,  stop = nchar(x)-nchar("_1.txt")))
myRwFiles <- merge(myRwFiles, myStrngFle, by='tag')
print(myRwFiles)

myRwFrmnts <- do.call(rbind.data.frame,
                      lapply(myRwFiles[doUse=='Y' & doSweep=='Y',]$fleNme, FUN=function(fn) {
                          # --- loop inbound formant files
                          print(paste0("...going for formant file >",fn,"<..."))
                          tmp <- fread(file=fn, sep="\t", quote="", header=TRUE, na.strings='--undefined--', dec='.')
                          print(paste0("... ...found ", VJtidyNumber(nrow(tmp)), " lines of data."))
                          tmp$GndrUse = myRwFiles[fleNme==fn,]$GndrUse
                          return(tmp)
                      })
)
print(paste0("Found in total ", VJtidyNumber(nrow(myRwFrmnts)), " lines of data in ", myRwFiles[doUse=='Y',.N], " files for sweep process."))

if(myCnfg$mdloCndtn>0) {
  tmp <- unique(myRwFrmnts$tf)
  print(paste0("Modulo condition filter found. Only the following ceiling frequencies modulo ", myCnfg$mdloCndtn, " == 0 will be processed!"))
  print(paste(tmp[which(tmp %% myCnfg$mdloCndtn == 0)], collapse = ','))
  myRwFrmnts <- myRwFrmnts[tf %% myCnfg$mdloCndtn == 0]
  print(paste0("Records reduced to ", VJtidyNumber(myRwFrmnts[,.N]), " lines of data."))
}

print(paste0("..breakdown totals by tag:"))
print(myRwFrmnts[,.(lines=VJtidyNumber(.N)),by=tag])
print(paste0("..breakdown by tag and frequency ceiling:"))
print(dcast(summaryBy(tag~tag+tf, myRwFrmnts, FUN=function(x) c(N=VJtidyNumber(length(x)))), tag~tf, value.var='tag.N'))
print('')

setwd(myCnfg$BseDrctry)

# === in case strip sample
myRwFrmnts[, sampleOrg := sample]
if (myCnfg$RgxSmple != '.+') {myRwFrmnts[, sample := regmatches(sample, regexpr(myCnfg$RgxSmple, sample))]}

# === some preparations
myRwFrmnts[, myKeySample := paste(tag,sampleID,sep='~')]
myRwFrmnts[, myKeySweep := paste(tag,sampleID,tf,sep='~')]
myRwFrmnts[, 'i_' := nrow(myRwFrmnts):1]
myRwFrmnts[, samplecontext := paste0(previoussample,'>',sample,'<',nextsample)]

# === get VJ tab 59
setwd(myCnfg$VJDrctry)
myFstats <- fread(file="data\\VJtab59.txt")
myFstats[, c('sample', 'GndrUse') := .(phoneme,gender)]
myRwFrmnts <- merge(myRwFrmnts, myFstats, by=c('sample','GndrUse'), all.x=TRUE)

myPrmtrs <- with(myCnfg, list(
  myCushion=SwpCshn, 
  myMaxOrder=OrdrDTT, 
  myWeightBandwith=wgtBndWdth, 
  myPickSingle=PckPntSngle, 
  myPickPair=PckPntPr, 
  myTriangleTest=TrngleTst,
  myTriangleCreep=TrngleCrp, 
  myGenderHintMalus = GndrHntMls, 
  myF3errorWeight=F3errWgt, 
  myLocationProbabilityWeight=LctnPrbbltyWght, 
  myLocationProbabilityWidth=LctnPrbbltyWdth
  ))

setwd(myCnfg$BseDrctry)
if(myCnfg$RprtMde=='process') {
  # === standard
  print(paste0("=== Starting full VJ sweep..."))

  myTmeStr = Sys.time()
  if (myCnfg$PrlllDo=='no' | myCnfg$PrlllJbs==1) {
    # --- we go single core processing
    print(paste0("Starting single core processing..."))
    tmpRsltSrs <- with(myCnfg, myRwFrmnts[order(i_, decreasing = TRUE), do.call(VJformantDTT, c(list(.SD, signOfLive=1000), myPrmtrs)), by=myKeySweep]) # the series
    tmpRsltWnnr <- tmpRsltSrs[, VJmarkWinner(.SD, selectBand=myCnfg$AvrgngBnd), by=myKeySample]
    RsltFnl <- tmpRsltWnnr[, VJextractWinner(.SD), by=myKeySample]
  } else {
    # --- we go parallel processing
    print(paste0("Starting ", myCnfg$PrlllJbs, " core processing..." ))
    require(doSNOW)
    require(foreach)
    myClsters <- makeCluster(myCnfg$PrlllJbs)
    registerDoSNOW(myClsters)
    
    # --- split data in chunks to distribute on CPUs
    myRwFrmnts[,mySpltID := as.numeric(as.factor(myRwFrmnts$myKeySample)) %% myCnfg$PrlllJbs]
    tmp <- myRwFrmnts[,.N,by=mySpltID]
    print(paste0("Running sweep on ",myCnfg$PrlllJbs," cores with max ", VJtidyNumber(max(tmp$N)), " samples each in total silence. Be patient and watch your CPU load in the task manager..."))
    
    # --- start parallel processing (silent)
    RsltFnl <- foreach (mySplt = 0:(myCnfg$PrlllJbs-1), .packages=c('data.table','hqmisc','dtt'), .combine=rbind) %dopar% {
      tmpRsltSrs <- with(myCnfg, myRwFrmnts[mySpltID==mySplt, do.call(VJformantDTT, c(list(.SD), myPrmtrs)), by=myKeySweep]) 
      tmpRsltWnnr <- tmpRsltSrs[, VJmarkWinner(.SD, selectBand=myCnfg$AvrgngBnd), by=myKeySample]
      tmpRsltFnl <- tmpRsltWnnr[, VJextractWinner(.SD), by=myKeySample]
      return(tmpRsltFnl)
    }
    print(paste0("Stopping parallel nodes."))
    stopCluster(myClsters)
  }
  print(paste0("Job ended after ", VJtidyNumber(as.double(difftime(Sys.time(),myTmeStr,units='mins')))," minutes."))

  # --- saving formants
  print("Saving results...")
  RsltFnl[,VJmyNamedSave(.SD, tag=.BY, lead='VJformantsSwept', directory=myCnfg$OutDrctry),by=tag]
  
  # --- write some statistics
  print("Saving statistics...")
  mySts <- function(x, lead='') {
    tmp <- signif(quantile(x, probs=c(0.1,0.25,0.5,0.75,0.9), na.rm=TRUE),3)
    names(tmp) <- paste0(lead,'Q',c('10','25','50','75','90'))
    return(as.list(tmp))
  }
  StsRsltFnl <- RsltFnl[, c(N=.N, mySts(F1P, lead='F1P'), mySts(F2P, lead='F2P'), mySts(F3P, lead='F3P')), by=.(tag,sample)]
  StsRsltFnl[order(sample),VJmyNamedSave(.SD, tag=.BY, lead='VJformantsSwept_Stats', directory=myCnfg$RprtDrctry), by=tag]
  
} else if( with(myCnfg, RprtMde=='SelectedWithPictures' | RprtMde=='SelectedWithPicturesAndData' && RprtSmpleIDStrt>0 )) {
  # === debug mode
  print(paste0("=== Starting selected VJ sweep for samples ", myCnfg$RprtSmpleIDStrt, " to ", myCnfg$RprtSmpleIDEnd, "..."))
  setwd(myCnfg$RprtDrctry)
  
  for (mySmpleID in with(myCnfg, RprtSmpleIDs)) {
    # --- loop all phonem IDs provided
    if (myRwFrmnts[sampleID==mySmpleID, .N] > 0 ) {
      # mySmpleID <-   310970		            
      message(mySmpleID, ' ', appendLF = FALSE)
      
      RsltSrs <- with(myCnfg, myRwFrmnts[sampleID==mySmpleID, do.call(VJformantDTT, c(list(.SD, returnSeries=TRUE), myPrmtrs)), by=myKeySweep]) # the series
      RsltWnnr <- with(myCnfg, VJmarkWinner(myRwFrmnts[sampleID==mySmpleID, do.call(VJformantDTT, c(list(.SD), myPrmtrs)), by=myKeySweep], selectBand=myCnfg$AvrgngBnd))
      RsltFnl <- VJextractWinner(RsltWnnr)
      
      # enrich the series with info on the selected curves
      RsltSrs <- merge(RsltSrs, RsltWnnr[,.(myKeySweep,FrmntSlct)], by='myKeySweep', all.x=TRUE)
      
      # === plot sweep
      dub <- melt(RsltSrs, id.vars = c('tRel','tf','pckP','pckL','pckR','FltErr','FrmntSlct'), measure.vars = c('F1', 'F2', 'F3', 'F1Fltrd', 'F2Fltrd', 'F3Fltrd'), variable.name = 'formant')
      dub$myClr <- with(dub, ifelse(formant=='F1' | formant=='F2' | formant=='F3' | FrmntSlct == 0, 'na', paste(formant)))
      pckUps <- subset(dub, (pckP==1 | pckL==1 | pckR==1) & (formant=='F1Fltrd' | formant=='F2Fltrd' | formant=='F3Fltrd'))
      pckUpsP <- subset(pckUps, pckP==1)
      pckUpsErr <- subset(pckUps, pckP==1 &  formant=='F1Fltrd')
      pckUpsErr$myClr <- ifelse(pckUpsErr$FrmntSlct>=1, 'F1Fltrd', 'na')
      myMid <- c(max(RsltSrs$tRel)*1000, (max(RsltSrs$F3) - min(RsltSrs$F1))/1.5)
      #mySample <- with( RsltSrs[1,], paste0(tag,' samp ',sampleID,' ', sample, ' in an ',annotationID,' ', annotation,' t=', t*1000))
      mySample <- with( RsltSrs[1,], paste0(tag,' ',sampleID))
      
      print(ggplot(data=dub, aes(x=tRel*1000, y=value, shape=formant, color=myClr)) +
              theme_bw() +
              scale_shape_manual(values=c(0,0,4,4,2,2)) +
              scale_color_manual(values=c('red','darkgreen','blue','black')) +
              theme(legend.position='none') +
              geom_point(data=subset(dub, formant=='F1' | formant=='F2' | formant=='F3'), size=3) +
              geom_line(data=subset(dub, formant=='F1Fltrd' | formant=='F2Fltrd' | formant=='F3Fltrd'), size=2) +
              geom_point(data=pckUps, size=5) +
              geom_hline(data=pckUpsP, aes(yintercept=value)) +
              geom_vline(data=pckUpsErr, aes(xintercept=tRel*1000), color='black') +
              geom_text(data=pckUpsP, aes(label=paste0(signif(value,3),' Hz')), vjust=2, hjust=-0.2) +
              geom_text(data=pckUpsErr, aes(label=paste0(signif(FltErr,3),' db'), x=myMid[1], y=myMid[2]), hjust=1.2) +
              xlab('intra sample time [msec]') +
              ylab('frequency [Hz]') +
              facet_wrap(~tf) +
              scale_y_log10() +
              ggtitle(mySample))
      tmp <- paste0('VJsw1 ',mySample,'.png')
      ggsave(filename = tmp, units='cm', width=29.7, height=21, dpi = 150)
      
      # === plot winner
      dub <- melt(RsltSrs[FrmntSlct==1], id.vars = c('tRel','tf','pckP','pckL','pckR','FltErr','FrmntSlct'), measure.vars = c('F1', 'F2', 'F3', 'F1Fltrd', 'F2Fltrd', 'F3Fltrd'), variable.name = 'formant')
      dub$myClr <- with(dub, ifelse(formant=='F1' | formant=='F2' | formant=='F3' | FrmntSlct == 0, 'na', paste(formant)))
      RsltFnl$formant = 'F1' # dummy
      print(ggplot(data=dub, aes(x=tRel*1000, y=value, shape=formant, color=myClr)) +
              theme_bw() +
              scale_shape_manual(values=c(0,0,4,4,2,2)) +
              scale_color_manual(values=c('red','darkgreen','blue','black')) +
              theme(legend.position='none') +
              geom_point(data=subset(dub, formant=='F1' | formant=='F2' | formant=='F3'), size=4) +
              geom_line(data=subset(dub, formant=='F1Fltrd' | formant=='F2Fltrd' | formant=='F3Fltrd'), size=2) +
              geom_point(data=RsltFnl, aes(y=F1P), shape=15, size=7, color='red') +
              geom_point(data=RsltFnl, aes(y=F2P), shape=16, size=7, color='darkgreen') +
              geom_point(data=RsltFnl, aes(y=F3P), shape=17, size=7, color='blue') +
              geom_vline(data=RsltFnl, aes(xintercept=tRel*1000), color='black') +
              geom_text(data=RsltFnl, aes(label=paste0('~' ,signif(F1P,3),' Hz'), y=F1P), vjust=2, hjust=-0.2, color='red') +
              geom_text(data=RsltFnl, aes(label=paste0('~' ,signif(F2P,3),' Hz'), y=F2P), vjust=2, hjust=-0.2, color='darkgreen') +
              geom_text(data=RsltFnl, aes(label=paste0('~' ,signif(F3P,3),' Hz'), y=F3P), vjust=2, hjust=-0.2, color='blue') +
              geom_text(data=RsltFnl, aes(label=paste0('Err ~', signif(FltErr,3),' db'), y=(F2P+F3P)/2), color='black', hjust=1.2) +
              geom_text(data=RsltFnl, aes(label=paste0('N=',FrmntSlctN,', opt. ceil. ', tf, ' Hz'), y=(F2P+F3P)/2), color='black', hjust=-0.2) +
              xlab('intra sample time [msec]') +
              ylab('frequency [Hz]') +
              scale_y_log10() +
              ggtitle(mySample))
      ggsave(filename = paste0('VJsw2 ',mySample,'.png'), units='cm', width=29.7, height=21, dpi = 150)
      
      # === in case save data as well
      if (myCnfg$RprtMde=='picturesAndData') {
        fwrite(RsltSrs, file = paste0('VJformantsSwept_RsltSrs ',mySample,'.txt'), sep="\t")
        fwrite(RsltWnnr, file = paste0('VJformantsSwept_RsltWnnr ',mySample,'.txt'), sep="\t")
        fwrite(RsltFnl, file = paste0('VJformantsSwept_RsltFnl ',mySample,'.txt'), sep="\t")
      }
    }
  }
} else {
   print(paste0("ERROR. Don't know what to do with this report mode: >",myCnfg$RprtMde,"<"))
 }
# === plot sweep

